#INCLUDE "dcdialog.CH"
#INCLUDE "dcgraph.CH"

#if XPPVER > 1900000

CLASS DC_XbpRmChart FROM DC_XbpRMChartSuper, DC_XbaseParts

EXPORTED:
  VAR cargo
  VAR font
  VAR regions
  VAR toolTipWindow
  VAR toolTipText
  VAR toolTipGetList
  VAR toolTipData
  VAR toolTipEnable
  VAR toolTipFont
  VAR toolTipFontHeight
  VAR toolTipTextBlock
  VAR toolTipTextColorBlock
  VAR toolTipWindowColorBlock
  VAR toolTipTextObject
  METHOD init, create, WhenHideEval, ApplyData, ApplyDataAxis, ApplyLabelAxis, ;
         ApplyLegend, ApplyColors, ShowToolTip, ToolTipTextDefault

ENDCLASS

* ---------------------

METHOD DC_XbpRmChart:init( oParent, oOwner, aPos, aSize, aPres, lVisible, oGetList )

LOCAL xOptions, cCaption, bBlock, lAutoSize := .f., ;
      aGetList, aGetListItem, cClsID

::DC_XbaseParts:init( oGetList,,self )

aGetList := ::getList:getListArray
aGetListItem := ::getList:getListArray[::getListPointer]

::toolTipEnable := .t.
::toolTipGetList := {}
::toolTipFont := '12.Lucida Console'
::toolTipFontHeight := 13.2
::toolTipTextBlock := {|a,b,c|::toolTipTextDefault(a,b,c)}
::toolTipTextColorBlock := {|a,b,c|{GRA_CLR_BLACK,GRA_CLR_YELLOW}}
::toolTipWindowColorBlock := {||GRA_CLR_YELLOW}

IF Empty(aPos)
  aPos := { 1, 1 }
ENDIF

IF Empty(aSize) .OR. Empty(aSize[1]) .OR. Empty(aSize[2])
  aSize := { oParent:currentSize()[1], oParent:currentSize()[2] }
ENDIF

::XbpActiveXControl:init( oParent, oOwner, aPos, aSize, aPres, lVisible )

::clsID := 'RMChart.RMChartX'
::regions := {}

::cargo := { ::getListPointer, aGetList, aGetListItem[xGETLIST_CARGO] }

::font := aGetListItem[cGETLIST_FONT]
DEFAULT ::font := "Comic Sans MS"

bBlock := DC_GetBlock(aGetListItem[bGETLIST_GROUP],'O')

IF Valtype(bBlock) = 'B'
  Eval( bBlock, self )
ENDIF

bBlock := DC_GetBlock(aGetListItem[bGETLIST_PREEVAL])
IF Valtype(bBlock) = 'B'
  Eval( bBlock, self )
ENDIF

RETURN self

* ------------------

METHOD DC_XbpRMChart:create( oParent, oOwner, aPos, aSize, aPres, lVisible )

::xbpActiveXControl:create( oParent, oOwner, aPos, aSize, aPres, lVisible )

::DC_XbaseParts:setOrig(self)

::reset()

RETURN self

* ---------------------------

METHOD DC_XbpRMChart:WhenHideEval( lEnable, lShow, lProtect )

IF Valtype(::whenBlock) = 'B'
  lEnable:= Eval(::whenBlock,self)
  IF lEnable
    ::enable()
  ELSE
    ::disable()
  ENDIF
ENDIF

IF Valtype(::hideBlock) = 'B'
  lShow := !Eval(::hideBlock,self)
  IF lShow
    IF !::isVisible()
      ::show()
    ENDIF
  ELSE
    IF ::isVisible()
      ::hide()
    ENDIF
  ENDIF
ENDIF

RETURN self

* --------------

METHOD DC_XbpRMChart:ApplyData( aData, nRegion, nGraphType, nWhich )

LOCAL oObject, cString := '', i

DEFAULT nRegion := 1, ;
        nWhich := 1, ;
        nGraphType := DCRMCHART_GRAPHTYPE_BARSERIES

IF nGraphType == DCRMCHART_GRAPHTYPE_BARSERIES
   oObject := ::Region(nRegion):BarSeries(nWhich)
ELSEIF nGraphType == DCRMCHART_GRAPHTYPE_LINESERIES
   oObject := ::Region(nRegion):LineSeries(nWhich)
ELSEIF nGraphType == DCRMCHART_GRAPHTYPE_GRIDLESSSERIES
   oObject := ::Region(nRegion):GridlessSeries()
ENDIF

IF Valtype(oObject) == 'O'
  FOR i := 1 TO Len(aData)
    cString += Alltrim(Str(aData[i])) + IIF(i<Len(aData),'*','')
  NEXT
  oObject:dataString := cString
ENDIF

RETURN Valtype(oObject) == 'O'

* --------------

METHOD DC_XbpRMChart:ApplyDataAxis( aData, nRegion, nWhich )

LOCAL oObject, i, cString := ''

DEFAULT nRegion := 1, ;
        nWhich := 1

oObject := ::Region(nRegion):DataAxis(nWhich)

IF Valtype(oObject) == 'O'
  FOR i := 1 TO Len(aData)
    cString += Alltrim(aData[i]) + IIF(i<Len(aData),'*','')
  NEXT
  oObject:LabelString := cString
ENDIF

RETURN Valtype(oObject) == 'O'

* --------------

METHOD DC_XbpRMChart:ApplyLabelAxis( aData, nRegion )

LOCAL oObject, i, cString := ''

DEFAULT nRegion := 1

oObject := ::Region(nRegion):LabelAxis()

IF Valtype(oObject) == 'O'
  FOR i := 1 TO Len(aData)
    cString += Alltrim(aData[i]) + IIF(i<Len(aData),'*','')
  NEXT
  oObject:LabelString := cString
ENDIF

RETURN Valtype(oObject) == 'O'

* --------------

METHOD DC_XbpRMChart:ApplyLegend( aData, nRegion )

LOCAL oObject, i, cString := ''

DEFAULT nRegion := 1

oObject := ::Region(nRegion):Legend

IF Valtype(oObject) == 'O'
  FOR i := 1 TO Len(aData)
    cString += Alltrim(DC_XtoC(aData[i])) + IIF(i<Len(aData),'*','')
  NEXT
  oObject:LegendString := cString
ENDIF

RETURN Valtype(oObject) == 'O'

* --------------

METHOD DC_XbpRMChart:ApplyColors( aColors, nRegion, nGraphType, nWhich )

LOCAL oObject, cString := '', i

DEFAULT nRegion := 1, ;
        nWhich := 1, ;
        nGraphType := DCRMCHART_GRAPHTYPE_BARSERIES

IF nGraphType == DCRMCHART_GRAPHTYPE_BARSERIES
   oObject := ::Region(nRegion):BarSeries(nWhich)
ELSEIF nGraphType == DCRMCHART_GRAPHTYPE_LINESERIES
   oObject := ::Region(nRegion):LineSeries(nWhich)
ELSEIF nGraphType == DCRMCHART_GRAPHTYPE_GRIDLESSSERIES
   oObject := ::Region(nRegion):GridlessSeries()
ENDIF

IF Valtype(oObject) == 'O'
  FOR i := 1 TO Len(aColors)
    oObject:setColorValue(i,aColors[i])
  NEXT
ENDIF

RETURN Valtype(oObject) == 'O'

* --------------

METHOD DC_XbpRMChart:ShowToolTip( nMouseButton, nX, nY, aData )

LOCAL GetList, GetOptions, nSeriesIndex, nDataIndex, nRegion, oRegion, aPos, ;
      lStatus := .f., nNumLines, nHeight, aColor, nColor, nSeconds, aPos1

IF !::toolTipEnable
  RETURN nil
ENDIF

IF Valtype(aData) == 'O' .AND. aData:isDerivedFrom('VtRecord')
  aData := aData:value
ENDIF

IF nMouseButton == -1 .AND. !Empty(::toolTipWindow)
  ::toolTipWindow := nil
  ::toolTipData := nil
  ::toolTipText := nil
  ASize(::toolTipGetList,0)
  RETURN .f.
ENDIF

nRegion := aData[5,2]
nSeriesIndex := aData[10,2]
nDataIndex := aData[11,2]

IF nSeriesIndex == 0 .OR. nDataIndex == 0 .OR. nRegion == 0
  IF Valtype(::toolTipWindow) == 'O'
    ::toolTipWindow:hide()
  ENDIF
  RETURN nil
ENDIF

#if XPPVER > 1900345
nSeconds := Seconds()
aPos := GetPointerPos()
IF Empty(::toolTipWindow) .OR. !::toolTipWindow:isVisible()
  DO WHILE Seconds() - nSeconds < .3
    aPos1 := GetPointerPos()
    IF aPos1[1] # aPos[1] .OR. aPos1[2] # aPos[2]
      RETURN nil
    ENDIF
    Sleep(10)
  ENDDO
ENDIF
#endif

oRegion := ::regions[nRegion]
::toolTipText := Eval(::toolTipTextBlock,nSeriesIndex,nDataIndex,oRegion)

nNumLines := DC_NumAt(";",::toolTipText) + 1
::toolTipText := Strtran(::toolTipText,';',Chr(13))

GetList := ::toolTipGetList

lStatus := .t.
IF Empty(::toolTipData)

  nColor := Eval(::toolTipWindowColorBlock)
  ::toolTipData := aData
  @ 0,0 DCSAY {||::toolTipText} OBJECT ::toolTipTextObject ;
        SAYSIZE 300, 90 SAYLEFT FONT '10.Lucida Console Bold' ;
        COLOR Eval(::toolTipTextColorBlock) SAYWORDBREAK
  @ 100,0 DCSAY {||Alltrim(Str(::toolTipData[25,2],9,2))} SAYSIZE 244,20 SAYLEFT ;
          FONT '14.Lucida Console Bold'
  DCGETOPTIONS HIDE ORIGIN XBPDLG_ORIGIN_MOUSE NOTITLEBAR ;
     COLOR nColor PIXEL _RESIZE .f. _AUTORESIZE .f.
  DCREAD GUI FIT EXIT OPTIONS GetOptions EVAL {|o|::toolTipWindow := o} SAVE
  Sleep(5)

ELSE

  aColor := Eval(::toolTipTextColorBlock,nSeriesIndex,nDataIndex,oRegion)

  ::toolTipData[25,2] := aData[25,2]
  DC_GetRefresh(GetList)
  IF Valtype(::toolTipWindow) == 'O'

    nHeight := nNumLines * ::toolTipFontHeight

    ::toolTipTextObject:setSize({::toolTipTextObject:currentSize()[1],nHeight})
    ::toolTipWindow:setSize({::toolTipWindow:currentSize()[1], ;
        ::toolTipTextObject:currentSize()[2]+::toolTipTextObject:currentPos()[2]+15})

    ::toolTipTextObject:setColorFG(aColor[1])
    ::toolTipTextObject:setColorBG(aColor[2])

    aPos  := DC_CalcAbsolutePosition({nX,nY},self)
    aPos[1] += 10
    aPos[2] -= ::toolTipWindow:currentSize()[2] + 40
    ::toolTipWindow:setPos(aPos)
    ::toolTipWindow:show()

  ENDIF
  Sleep(1)

ENDIF

RETURN lStatus

* ---------------

METHOD DC_XbpRMChart:toolTipTextDefault( nSeriesIndex, nDataIndex, oRegion )

LOCAL cToolTipText

IF nDataIndex > 0
  IF !Empty(oRegion:labelAxisLabelText)
    cToolTipText := oRegion:labelAxisLabelText[nDataIndex]
  ELSE
    cToolTipText := oRegion:legendText[nDataIndex]
  ENDIF
ELSE
  cToolTipText := ''
ENDIF

RETURN cToolTipText

* ============================================================

CLASS DC_XbpRmChartRegion FROM  DC_XbpRmChartRegionSuper, DC_XbaseParts

EXPORTED:
  VAR region
  VAR cargo
  VAR parent
  VAR pos
  VAR size
  VAR footer
  VAR captionTitle
  VAR captionFontSize
  VAR captionBold
  VAR captionBackColor
  VAR captionTextColor
  VAR hasGrid
  VAR hasLegend
  VAR hasDataAxis
  VAR hasLabelAxis
  VAR hasBarGroup
  VAR hasLineGroup
  VAR hasGridlessGroup
  VAR gridAsGradient
  VAR gridBackColor
  VAR gridBiColorMode
  VAR dataAxis
  VAR labelAxisAlign
  VAR labelAxisText
  VAR labelAxisCount
  VAR labelAxisTickCount
  VAR labelAxisFontSize
  VAR labelAxisLabelText
  VAR labelAxisLineColor
  VAR labelAxisLineStyle
  VAR labelAxisTextColor
  VAR labelAxisTextAlign
  VAR legendAlign
  VAR legendBackColor
  VAR legendTextColor
  VAR legendBold
  VAR legendFontSize
  VAR legendStyle
  VAR legendText
  VAR barGroup
  VAR lineGroup
  VAR gridlessGroup

  METHOD init, Reset

ENDCLASS

* ------------------

METHOD DC_XbpRmChartRegion:Init( oParent, aPos, aSize, oGetList )

LOCAL aGetList, aGetListItem, nRegion, aCaption, aGrid, aDataAxis, ;
      aLabelAxis, aBarGroup, aLineGroup, aGridlessGroup, aMisc, ;
      aLegend, i, bBlock

::DC_XbaseParts:init( oGetList,,self )

aGetList := ::getList:getListArray
aGetListItem := ::getList:getListArray[::getListPointer]

::parent := oParent
::pos := aPos
::size := aSize

::parent:AddRegion()

AAdd(::parent:regions,self)
::region := Len(::parent:regions)

* --- Caption ---
::parent:Region(::region):AddCaption()
aCaption := aGetListItem[cGETLIST_CAPTION]
DEFAULT aCaption := {}
ASize(aCaption,5)
::captionTitle := aCaption[1] ; DEFAULT ::captionTitle := ''
::captionFontSize := aCaption[2] ; DEFAULT ::captionFontSize := 11
::captionBold := aCaption[3] ; DEFAULT ::captionBold := .f.
::captionBackColor := aCaption[4]
::captionTextColor := aCaption[5]

* --- Misc ---
aMisc := aGetListItem[xGETLIST_OPTIONS]
DEFAULT aMisc := {}
ASize(aMisc,1)
::footer := aMisc[1] ; DEFAULT ::footer := ''

* --- Legend ---
aLegend := aGetListItem[xGETLIST_OPTIONS8]
::hasLegend := Valtype(aLegend) == 'A'
IF ::hasLegend
  ::parent:Region(::region):AddLegend()
ENDIF
DEFAULT aLegend := {}
ASize(aLegend,7)
::legendAlign := aLegend[1] ; DEFAULT ::legendAlign := RMC_LEGEND_TOP
::legendBackColor := aLegend[2] ; DEFAULT ::legendBackColor := LightYellow
::legendTextColor := aLegend[3] ; DEFAULT ::legendTextColor := Black
::legendBold := aLegend[4] ; DEFAULT ::legendBold := .f.
::legendFontSize := aLegend[5] ; DEFAULT ::legendFontSize := 8
::legendStyle := aLegend[6] ; DEFAULT ::legendStyle := RMC_LEGENDRECT
::legendText := aLegend[7]

* --- Grid ---
aGrid := aGetListItem[xGETLIST_OPTIONS2]
::hasGrid := Valtype(aGrid) == 'A'
IF ::hasGrid
  ::parent:Region(::region):AddGrid()
ENDIF
DEFAULT aGrid := {}
ASize(aGrid,3)
::gridAsGradient := aGrid[1] ; DEFAULT ::gridAsGradient := .f.
::gridBackColor := aGrid[2]
::gridBiColorMode := aGrid[3] ; DEFAULT ::gridBiColorMode := RMC_BICOLOR_LABELAXIS

* --- Data Axis ---
::dataAxis := aGetListItem[xGETLIST_OPTIONS3]
::hasDataAxis := Valtype(::dataAxis) == 'A'
IF ::hasDataAxis
  FOR i := 1 TO Len(::dataAxis)
    ::parent:Region(::Region):AddDataAxis()
    ASize(::dataAxis[i],12)
    DEFAULT ::dataAxis[i,1] := RMC_DATAAXISLEFT
    DEFAULT ::dataAxis[i,2] := ''
    DEFAULT ::dataAxis[i,3] := ''
    DEFAULT ::dataAxis[i,4] := 8
    DEFAULT ::dataAxis[i,6] := Black
    DEFAULT ::dataAxis[i,7] := RMC_LINESTYLEDOT
    DEFAULT ::dataAxis[i,8] := 0
    DEFAULT ::dataAxis[i,9] := 100
    DEFAULT ::dataAxis[i,10] := 0
    DEFAULT ::dataAxis[i,11] := Black
    DEFAULT ::dataAxis[i,12] := IIF(Valtype(::dataAxis[i,5])=='A',Len(::dataAxis[i,5]),0)
  NEXT
ENDIF

* --- Label Axis ---
aLabelAxis := aGetListItem[xGETLIST_OPTIONS4]

::hasLabelAxis := Valtype(aLabelAxis) == 'A'
IF ::hasLabelAxis
  ::parent:Region(::region):AddLabelAxis()
  DEFAULT aLabelAxis := {}
  ASize(aLabelAxis,10)
  ::labelAxisAlign := aLabelAxis[1] ; DEFAULT ::labelAxisAlign := RMC_LABELAXISBOTTOM
  ::labelAxisText := aLabelAxis[2] ; DEFAULT ::labelAxisText := ''
  ::labelAxisCount := aLabelAxis[3] ; DEFAULT ::labelAxisCount := 1
  ::labelAxisLabelText := aLabelAxis[6]
  ::labelAxisTickCount := aLabelAxis[4] ; DEFAULT ::labelAxisTickCount := Len(::labelAxisLabelText)
  ::labelAxisFontSize := aLabelAxis[5] ; DEFAULT ::labelAxisFontSize := 8
  ::labelAxisLineColor := aLabelAxis[7] ; DEFAULT ::labelAxisLineColor := Black
  ::labelAxisLineStyle := aLabelAxis[8] ; DEFAULT ::labelAxisLineStyle := RMC_LINESTYLENONE
  ::labelAxisTextColor := aLabelAxis[9] ; DEFAULT ::labelAxisTextColor := Black
  ::labelAxisTextAlign := aLabelAxis[10] ; DEFAULT ::labelAxisTextAlign := RMC_TEXTCENTER
ENDIF

* --- Bar Group ---
::barGroup := aGetListItem[xGETLIST_OPTIONS5]
::hasBarGroup := Valtype(::barGroup) == 'A'
IF ::hasBarGroup
  FOR i := 1 TO Len(::barGroup)
    ::parent:Region(::region):AddBarSeries()
    ASize(::barGroup[i],10)
    IF Empty(::barGroup[i,2])
      ::barGroup[i,2] := CornflowerBlue
    ENDIF
    DEFAULT ::barGroup[i,3] := RMC_HATCHBRUSH_OFF
    DEFAULT ::barGroup[i,4] := .f.
    DEFAULT ::barGroup[i,5] := .f.
    DEFAULT ::barGroup[i,6] := 1
    DEFAULT ::barGroup[i,7] := RMC_BAR_FLAT_GRADIENT2
    DEFAULT ::barGroup[i,8] := RMC_BARGROUP
    DEFAULT ::barGroup[i,9] := 0
    DEFAULT ::barGroup[i,10] := 1
  NEXT
ENDIF

* --- Line Group ---
::lineGroup := aGetListItem[xGETLIST_OPTIONS6]
::hasLineGroup := Valtype(::lineGroup) == 'A'
IF ::hasLineGroup
  FOR i := 1 TO Len(::lineGroup)
    ::parent:Region(::region):AddLineSeries()
    ASize(::lineGroup[i],11)
    DEFAULT ::lineGroup[i,2] := Black
    DEFAULT ::lineGroup[i,3] := RMC_HATCHBRUSH_OFF
    DEFAULT ::lineGroup[i,4] := .f.
    DEFAULT ::lineGroup[i,6] := RMC_LINE_FLAT
    DEFAULT ::lineGroup[i,7] := RMC_LINE
    DEFAULT ::lineGroup[i,8] := 0
    DEFAULT ::lineGroup[i,9] := 1
    DEFAULT ::lineGroup[i,10] := RMC_SYMBOL_POINT
    DEFAULT ::lineGroup[i,11] := RMC_LSTYLE_LINE_AREA
  NEXT
ENDIF

* --- Gridless Group ---
::gridlessGroup := aGetListItem[xGETLIST_OPTIONS7]
::hasGridlessGroup := Valtype(::gridlessGroup) == 'A'
IF ::hasGridlessGroup
  FOR i := 1 TO Len(::gridlessGroup)
    ::parent:Region(::region):AddGridlessSeries()
    ASize(::gridlessGroup[i],9)
    DEFAULT ::gridLessGroup[i,3] := RMC_HATCHBRUSH_OFF
    DEFAULT ::gridlessGroup[i,4] := RMC_FULL
    DEFAULT ::gridLessGroup[i,5] := .f.
    DEFAULT ::gridlessGroup[i,6] := RMC_PIE_3D
    DEFAULT ::gridlessGroup[i,7] := 0
    DEFAULT ::gridlessGroup[i,8] := 0
    DEFAULT ::gridlessGroup[i,9] := 2
  NEXT
ENDIF

bBlock := DC_GetBlock(aGetListItem[bGETLIST_GROUP],'O')

IF Valtype(bBlock) = 'B'
  Eval( bBlock, self )
ENDIF

bBlock := DC_GetBlock(aGetListItem[bGETLIST_PREEVAL])
IF Valtype(bBlock) = 'B'
  Eval( bBlock, self )
ENDIF

::reset()

RETURN self

* ------------------

METHOD DC_XbpRmChartRegion:Reset()

LOCAL nRegion, cString, i, j, nColor

nRegion := ::region

* ---  Region ---
::parent:region(nRegion):left := ::pos[1]
::parent:region(nRegion):top := ::pos[2]
::parent:region(nRegion):width := ::size[1]
::parent:region(nRegion):height := ::size[2]
::parent:region(nRegion):footer := ::footer

* ---  Caption ---
::parent:Region(nRegion):Caption():Titel := ::captionTitle
IF !Empty(::captionBackColor)
  ::parent:Region(nRegion):Caption():BackColor := ::captionBackColor
ENDIF
IF !Empty(::captionTextColor)
  ::parent:region(nRegion):Caption():TextColor := ::captionTextColor
ENDIF
::parent:Region(nRegion):Caption():FontSize := ::captionFontSize
::parent:Region(nRegion):Caption():Bold := ::captionBold

* --- Legend ---
IF ::hasLegend
  ::parent:Region(nRegion):Legend():Alignment := ::legendAlign
  ::parent:Region(nRegion):Legend():BackColor := ::legendBackColor
  ::parent:Region(nRegion):Legend():TextColor := ::legendTextColor
  ::parent:Region(nRegion):Legend():Style := ::legendStyle
  ::parent:Region(nRegion):Legend():Bold := ::legendBold
  ::parent:Region(nRegion):Legend():FontSize := ::legendFontSize
  IF Valtype(::legendText) == 'A'
    cString := ''
    FOR i := 1 TO Len(::legendText)
      cString += Alltrim(::LegendText[i]) + IIF(i<Len(::legendText),'*','')
    NEXT
    ::parent:Region(nRegion):Legend():LegendString := cString
  ENDIF
ENDIF

* --- Grid ---
IF ::hasGrid
  IF !Empty(::gridBackColor)
    ::parent:Region(nRegion):Grid():BackColor := ::gridBackColor
  ENDIF
  ::parent:Region(nRegion):Grid():AsGradient := ::gridAsGradient
  IF !Empty(::gridBiColorMode)
    ::parent:Region(nRegion):Grid():BicolorMode := ::gridBiColorMode
  ENDIF
  ::parent:Region(nRegion):Grid():Left := 0
  ::parent:Region(nRegion):Grid():Top := 0
  ::parent:Region(nRegion):Grid():Width := 0
  ::parent:Region(nRegion):Grid():Height := 0
ENDIF

* --- Data Axis ---
IF ::hasDataAxis
  FOR i := 1 TO Len(::dataAxis)
    ::parent:Region(nRegion):DataAxis(i):Alignment := ::dataAxis[i,DCRMCHART_DATAAXIS_ALIGNMENT]
    ::parent:Region(nRegion):DataAxis(i):MinValue := ::dataAxis[i,DCRMCHART_DATAAXIS_MINVALUE]
    ::parent:Region(nRegion):DataAxis(i):MaxValue := ::dataAxis[i,DCRMCHART_DATAAXIS_MAXVALUE]
    ::parent:Region(nRegion):DataAxis(i):TickCount := ::dataAxis[i,DCRMCHART_DATAAXIS_TICKCOUNT]
    ::parent:Region(nRegion):DataAxis(i):FontSize := ::dataAxis[i,DCRMCHART_DATAAXIS_FONTSIZE]
    ::parent:Region(nRegion):DataAxis(i):TextColor := ::dataAxis[i,DCRMCHART_DATAAXIS_TEXTCOLOR]
    ::parent:Region(nRegion):DataAxis(i):LineColor := ::dataAxis[i,DCRMCHART_DATAAXIS_LINECOLOR]
    ::parent:Region(nRegion):DataAxis(i):LineStyle := ::dataAxis[i,DCRMCHART_DATAAXIS_LINESTYLE]
    ::parent:Region(nRegion):DataAxis(i):DecimalDigits := ::dataAxis[i,DCRMCHART_DATAAXIS_DECIMALDIGITS]
    ::parent:Region(nRegion):DataAxis(i):AxisUnit := ::dataAxis[i,DCRMCHART_DATAAXIS_AXISUNIT]
    IF !Empty(::dataAxis[i,DCRMCHART_DATAAXIS_AXISTEXT])
      ::parent:Region(nRegion):DataAxis(i):AxisText := ::dataAxis[i,DCRMCHART_DATAAXIS_AXISTEXT]
    ENDIF
    IF Valtype(::dataAxis[i,DCRMCHART_DATAAXIS_AXISLABEL]) == 'A'
      cString := ''
      FOR j := 1 TO Len(::dataAxis[i,DCRMCHART_DATAAXIS_AXISLABEL])
        cString += Alltrim(::dataAxis[i,DCRMCHART_DATAAXIS_AXISLABEL,j]) + IIF(j<Len(::dataAxis[i,DCRMCHART_DATAAXIS_AXISLABEL]),'*','')
      NEXT
      ::parent:Region(nRegion):DataAxis(i):LabelString := cString
    ENDIF
  NEXT
ENDIF

* --- Label Axis ---
IF ::hasLabelAxis
  ::parent:Region(nRegion):LabelAxis():AxisCount := ::labelAxisCount
  ::parent:Region(nRegion):LabelAxis():TickCount := ::labelAxisTickCount
  ::parent:Region(nRegion):LabelAxis():Alignment := ::labelAxisAlign
  ::parent:Region(nRegion):LabelAxis():FontSize := ::labelAxisFontSize
  ::parent:Region(nRegion):LabelAxis():TextColor := ::labelAxisTextColor
  ::parent:Region(nRegion):LabelAxis():TextAlignment := ::labelAxisTextAlign
  ::parent:Region(nRegion):LabelAxis():LineColor := ::labelAxisLineColor
  ::parent:Region(nRegion):LabelAxis():LineStyle := ::labelAxisLineStyle
  IF Valtype(::labelAxisLabelText) == 'A'
    cString := ''
    FOR i := 1 TO Len(::labelAxisLabelText)
      cString += Alltrim(::labelAxisLabelText[i]) + IIF(i<Len(::labelAxisLabelText),'*','')
    NEXT
    ::parent:Region(nRegion):LabelAxis():LabelString := cString
  ENDIF
ENDIF

* --- Bar Series ---
IF ::hasBarGroup

  FOR i := 1 TO Len(::barGroup)
    ::parent:Region(nRegion):BarSeries(i):SeriesType := ::barGroup[i,DCRMCHART_BARSERIES_SERIESTYPE]
    ::parent:Region(nRegion):BarSeries(i):SeriesStyle := ::barGroup[i,DCRMCHART_BARSERIES_SERIESSTYLE]
    ::parent:Region(nRegion):BarSeries(i):Lucent := ::barGroup[i,DCRMCHART_BARSERIES_LUCENT]
    ::parent:Region(nRegion):BarSeries(i):Horizontal := ::barGroup[i,DCRMCHART_BARSERIES_HORIZONTAL]
    ::parent:Region(nRegion):BarSeries(i):WhichDataAxis := ::barGroup[i,DCRMCHART_BARSERIES_WHICHDATAAXIS]
    ::parent:Region(nRegion):BarSeries(i):ValueLabelOn := ::barGroup[i,DCRMCHART_BARSERIES_VALUELABELON]
    ::parent:Region(nRegion):BarSeries(i):PointsPerColumn := ::barGroup[i,DCRMCHART_BARSERIES_POINTSPERCOLUMN]
    ::parent:Region(nRegion):BarSeries(i):HatchMode := ::barGroup[i,DCRMCHART_BARSERIES_HATCHMODE]
    IF Valtype(::barGroup[i,2]) == 'A'
      FOR j := 1 TO Len(::barGroup[i,DCRMCHART_BARSERIES_COLOR])
        nColor := ::barGroup[i,DCRMCHART_BARSERIES_COLOR,j]
        ::parent:Region(nRegion):BarSeries(i):setColorValue(j,nColor)
      NEXT
    ELSE
      ::parent:Region(nRegion):BarSeries(i):Color := ::barGroup[i,DCRMCHART_BARSERIES_COLOR]
    ENDIF
    IF Valtype(::barGroup[i,DCRMCHART_BARSERIES_DATA]) == 'A'
      cString := ''
      FOR j := 1 TO Len(::barGroup[i,DCRMCHART_BARSERIES_DATA])
        cString += Alltrim(Str(::barGroup[i,DCRMCHART_BARSERIES_DATA,j])) + IIF(j<Len(::barGroup[i,DCRMCHART_BARSERIES_DATA]),'*','')
      NEXT
      ::parent:Region(nRegion):BarSeries(i):dataString := cString
    ENDIF
  NEXT
ENDIF

* --- Line Series ---
IF ::hasLineGroup
  FOR i := 1 TO Len(::lineGroup)
    ::parent:Region(nRegion):LineSeries(i):SeriesType := ::lineGroup[i,DCRMCHART_LINESERIES_SERIESTYPE]
    ::parent:Region(nRegion):LineSeries(i):SeriesStyle := ::lineGroup[i,DCRMCHART_LINESERIES_SERIESSTYLE]
    ::parent:Region(nRegion):LineSeries(i):LineStyle := ::lineGroup[i,DCRMCHART_LINESERIES_LINESTYLE]
    ::parent:Region(nRegion):LineSeries(i):Lucent := ::lineGroup[i,DCRMCHART_LINESERIES_LUCENT]
    ::parent:Region(nRegion):LineSeries(i):Color := ::lineGroup[i,DCRMCHART_LINESERIES_COLOR]
    ::parent:Region(nRegion):LineSeries(i):SymbolStyle := ::lineGroup[i,DCRMCHART_LINESERIES_SYMBOLSTYLE]
    ::parent:Region(nRegion):LineSeries(i):WhichDataAxis := ::lineGroup[i,DCRMCHART_LINESERIES_WHICHDATAAXIS]
    ::parent:Region(nRegion):LineSeries(i):ValueLabelOn := ::lineGroup[i,DCRMCHART_LINESERIES_VALUELABELON]
    ::parent:Region(nRegion):LineSeries(i):HatchMode := ::lineGroup[i,DCRMCHART_LINESERIES_HATCHMODE]
    IF Valtype(::lineGroup[i,DCRMCHART_LINESERIES_POINTSPERCOLUMN]) == 'A'
      cString := ''
      FOR j := 1 TO Len(::lineGroup[i,DCRMCHART_LINESERIES_POINTSPERCOLUMN])
        cString += Alltrim(Str(::lineGroup[i,DCRMCHART_LINESERIES_POINTSPERCOLUMN,j])) + ;
                   IIF(j<Len(::lineGroup[i,DCRMCHART_LINESERIES_POINTSPERCOLUMN]),'*','')
      NEXT
      ::parent:Region(nRegion):LineSeries(i):pointsPerColumnString := cString
    ENDIF
    IF Valtype(::lineGroup[i,DCRMCHART_LINESERIES_DATA]) == 'A'
      cString := ''
      FOR j := 1 TO Len(::lineGroup[i,DCRMCHART_LINESERIES_DATA])
        cString += Alltrim(Str(::lineGroup[i,1,j])) + IIF(j<Len(::lineGroup[i,1]),'*','')
      NEXT
      ::parent:Region(nRegion):LineSeries(i):dataString := cString
    ENDIF
  NEXT
ENDIF

// RMChartX1:Region(1):BarSeries(1):SetColorValue( 3, Red )     // Set the color for the third bar to Red)

* --- Gridless Series ---
IF ::hasGridlessGroup

  FOR i := 1 TO Len(::gridlessGroup)
    ::parent:Region(nRegion):GridlessSeries():SeriesStyle := ::gridlessGroup[i,DCRMCHART_GRIDLESSSERIES_SERIESSTYLE]
    ::parent:Region(nRegion):GridlessSeries():Alignment := ::gridlessGroup[i,DCRMCHART_GRIDLESSSERIES_ALIGNMENT]
    ::parent:Region(nRegion):GridlessSeries():ExplodeMode := ::gridlessGroup[i,DCRMCHART_GRIDLESSSERIES_EXPLODEMODE]
    ::parent:Region(nRegion):GridlessSeries():Lucent := ::gridlessGroup[i,DCRMCHART_GRIDLESSSERIES_LUCENT]
    ::parent:Region(nRegion):GridlessSeries():ValueLabelOn := ::gridlessGroup[i,DCRMCHART_GRIDLESSSERIES_VALUELABELON]
    ::parent:Region(nRegion):GridlessSeries():HatchMode := ::gridlessGroup[i,DCRMCHART_GRIDLESSSERIES_HATCHMODE]
    IF Valtype(::gridlessGroup[i,2]) == 'A'
      cString := ''
      FOR j := 1 TO Len(::gridlessGroup[i,DCRMCHART_GRIDLESSSERIES_COLOR])
        cString += Alltrim(DC_XtoC(::gridlessGroup[i,2,j])) + IIF(j<Len(::gridlessGroup[i,2]),'*','')
      NEXT
      ::parent:Region(nRegion):GridlessSeries():ColorString := cString
    ENDIF
    IF Valtype(::gridlessGroup[i,DCRMCHART_GRIDLESSSERIES_DATA]) == 'A'
      cString := ''
      FOR j := 1 TO Len(::gridlessGroup[i,DCRMCHART_GRIDLESSSERIES_DATA])
        cString += Alltrim(DC_XtoC(::gridlessGroup[i,DCRMCHART_GRIDLESSSERIES_DATA,j])) + IIF(j<Len(::gridlessGroup[i,DCRMCHART_GRIDLESSSERIES_DATA]),'*','')
      NEXT
      ::parent:Region(nRegion):GridlessSeries():DataString := cString
    ENDIF
  NEXT
ENDIF

RETURN self

#endif


